<?php

namespace App\Http\Controllers\Api\users;

use App\Http\Controllers\Controller;
use App\Models\CancelledBooking;
use App\Models\Doctor;
use App\Models\BookingMessage;
use App\Models\DoctorUserRate;
use App\Models\ObjectBooking;
use App\Models\ObjectDetails;
use App\Models\ObjectWeekDays;
use App\Models\TimeSlot;
use App\Models\User;
use App\Models\UserBookings;
use App\Models\UserDeviceToken;
use App\Models\Notifications;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Traits\IntegrationTrait;
use App\Traits\RepositoryTrait;
use App\Traits\Notifications as NotificationsTrait;
use App\Traits\Firebase;
use Illuminate\Support\Facades\DB;

class BookingController extends Controller
{
    use IntegrationTrait;
    use RepositoryTrait;
    use NotificationsTrait;
    use Firebase;

    public function getUserBookings(Request $request)
    {
        $todayDate = date('Y-m-d');
        $todayTime = date('H:i:s');
        $userId = Auth::user()->id;
        $isExist = 0;
        $bookingsArr = UserBookings::where("user_id", $userId)->get("reservation_record_id");
        $reservationRecordIdArr = [];
        for ($j = 0; $j < count($bookingsArr); $j++) {
            $reservationRecordIdArr[] = $bookingsArr[$j]["reservation_record_id"];
        }
        $dataArray = [];
        if ($request->status == "finished") {
            $bookings = ObjectBooking::whereIn("reservation_record_id", $reservationRecordIdArr)->where("is_cancelled", 0)->orderBy("date")->get();
            for($i=0; $i<count($bookings); $i++){
            $slot = TimeSlot::where("id", $bookings[$i]->slot_id)->first();
            if (($bookings[$i]->date < $todayDate) || ($bookings[$i]->date == $todayDate && $slot->time_to <= $todayTime)) {
                $bookingsArr[$i]["id"] = UserBookings::where("reservation_record_id", $bookings[$i]->reservation_record_id)->first()->id;
                $day = ObjectWeekDays::where("id", $slot->object_week_days_id)->first();
                $isExist = 1;
                $timeFrom = strtotime($slot->time_from);
                $timeTo = strtotime($slot->time_to);
                $newFormatFrom = date('H:i', $timeFrom);
                $newFormatTo = date('H:i', $timeTo);
                $bookingsArr[$i]["date"] = $bookings[$i]->date;
                $bookingsArr[$i]["slot_from"] = $newFormatFrom;
                $bookingsArr[$i]["slot_to"] = $newFormatTo;
                $bookingsArr[$i]["day"] = $request->lang == "ar" ? $day->week_day_ar_name : $day->week_day_en_name;
                $bookingsArr[$i]["is_online"] = $bookings[$i]->is_online;
                $doctor = Doctor::where("object_id", $bookings[$i]->object_id)->first();
                $bookingsArr[$i]["doctor_name"] = $doctor->full_name;
                $doctor = Doctor::where("object_id", $bookings[$i]["object_id"])->first();
                $imageData = $doctor->getFirstMediaFile("Doctors");
                $bookingsArr[$i]["doctor_image"] = $imageData != null ? env("APP_PATH") . $imageData["path"] . "/" . $imageData["filename"] : null;
                $dataArray[] = $bookingsArr[$i];
            }
            }
        }elseif ($request->status == "unfinished") {
            $bookings = ObjectBooking::whereIn("reservation_record_id", $reservationRecordIdArr)->where("is_cancelled", 0)->orderBy("date")->get();
            for($i=0; $i<count($bookings); $i++){
                $slot = TimeSlot::where("id", $bookings[$i]->slot_id)->first();
                if (($bookings[$i]->date > $todayDate) || ($bookings[$i]->date == $todayDate && $slot->time_to > $todayTime)) {
                    $bookingsArr[$i]["id"] = UserBookings::where("reservation_record_id", $bookings[$i]->reservation_record_id)->first()->id;
                    $day = ObjectWeekDays::where("id", $slot->object_week_days_id)->first();
                    $isExist = 1;
                    $timeFrom = strtotime($slot->time_from);
                    $timeTo = strtotime($slot->time_to);
                    $newFormatFrom = date('H:i', $timeFrom);
                    $newFormatTo = date('H:i', $timeTo);
                    $bookingsArr[$i]["date"] = $bookings[$i]->date;
                    $bookingsArr[$i]["slot_from"] = $newFormatFrom;
                    $bookingsArr[$i]["slot_to"] = $newFormatTo;
                    $bookingsArr[$i]["day"] = $request->lang == "ar" ? $day->week_day_ar_name : $day->week_day_en_name;
                    $bookingsArr[$i]["is_online"] = $bookings[$i]->is_online;
                    $doctor = Doctor::where("object_id", $bookings[$i]->object_id)->first();
                    $bookingsArr[$i]["doctor_name"] = $doctor->full_name;
                    $doctor = Doctor::where("object_id", $bookings[$i]["object_id"])->first();
                    $imageData = $doctor->getFirstMediaFile("Doctors");
                    $bookingsArr[$i]["doctor_image"] = $imageData != null ? env("APP_PATH") . $imageData["path"] . "/" . $imageData["filename"] : null;
                    $dataArray[] = $bookingsArr[$i];
                }
            }
            }elseif ($request->status == "cancelled") {
            $bookings = ObjectBooking::whereIn("reservation_record_id", $reservationRecordIdArr)->where("is_cancelled", 1)->orderBy("date")->get();
            for($i=0; $i<count($bookings); $i++){
                $bookingsArr[$i]["id"] = UserBookings::where("reservation_record_id", $bookings[$i]->reservation_record_id)->first()->id;
                $slot = TimeSlot::where("id", $bookings[$i]->slot_id)->first();
                $day = ObjectWeekDays::where("id", $slot->object_week_days_id)->first();
                $isExist = 1;
                $timeFrom = strtotime($slot->time_from);
                $timeTo = strtotime($slot->time_to);
                $newFormatFrom = date('H:i', $timeFrom);
                $newFormatTo = date('H:i', $timeTo);
                $bookingsArr[$i]["date"] = $bookings[$i]->date;
                $bookingsArr[$i]["slot_from"] = $newFormatFrom;
                $bookingsArr[$i]["slot_to"] = $newFormatTo;
                $bookingsArr[$i]["day"] = $request->lang == "ar" ? $day->week_day_ar_name : $day->week_day_en_name;
                $bookingsArr[$i]["is_online"] = $bookings[$i]->is_online;
                $doctor = Doctor::where("object_id", $bookings[$i]->object_id)->first();
                $bookingsArr[$i]["doctor_name"] = $doctor->full_name;
                $doctor = Doctor::where("object_id", $bookings[$i]["object_id"])->first();
                $imageData = $doctor->getFirstMediaFile("Doctors");
                $bookingsArr[$i]["doctor_image"] = $imageData != null ? env("APP_PATH") . $imageData["path"] . "/" . $imageData["filename"] : null;
                $dataArray[] = $bookingsArr[$i];
            }
        }
        if ($isExist == 1)
            return response()->json([
                "status" => true,
                "bookings" => $dataArray
            ]);
        else {
            $message = BookingMessage::where([["remark", 5], ["lang", $request->lang]])->first()->msg;
            return response()->json([
                "status" => true,
                "bookings" => $message
            ]);
        }
    }

    public function addBooking(Request $request)
    {
        /*$firebaseToken = ["c_m1xTQoShSELoyGGVzDam:APA91bFFdgyQ8_uK5hKmFbgtuXVe7ep7Ox6WyuIckKd00wM8TbfHjsAUCvyZ1B4hJL0ZxTTN3G2SEEKTFB5EUifMC3cwcgCuukQzB6ETE1XHH5H1Myds64C-dGTcuFu1ZXoq-5EyxSDH"];
        $SERVER_API_KEY = 'AAAAed6NnRY:APA91bF23zzJGo57bTTQUo77w1wvnAPTGWLR8vE92aNDk-UsPjazqPQBNj9XDZEo5VZ6uh43iBxNJkaYKwJqk1ZL9yGq-brQSkYibFEFRnL9pxB3kZa4lo3B8FuBwQYZmq-EeqNTXP_m';

        $data = [
            "registration_ids" => $firebaseToken,
            "notification" => [
                "title" => "Title",
                "body" => "Body",
            ]
        ];
        $dataString = json_encode($data);

        $headers = [
            'Authorization: key=' . $SERVER_API_KEY,
            'Content-Type: application/json',
        ];

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $dataString);

        $response = curl_exec($ch);

        dd($response);*/
dd("///222");
        return $this->addNewBooking($request);
    }

    public function getBookingDetails(Request $request)
    {
        return $this->getById($request);
    }
    public function getDoctorBookings(Request $request){
        return $this->doctorBookings($request);
    }
    public function getDoctorBookingDetails(Request $request)
    {
        $repository = $this->getRepository("ObjectBookingRepository");
        return $repository->getById($request);
    }
    public function cancelBooking(Request $request){
        DB::beginTransaction();
        try {
        $checkCancellation = $this->checkCancellation($request->id);
        if($checkCancellation["status"]) {
            $booking = ObjectBooking::find($request->id);
            $booking->is_cancelled = 1;
            $booking->save();
            $cancelledBooking = new CancelledBooking();
            $cancelledBooking->booking_id = $request->id;
            $cancelledBooking->cancelled_by = Auth::user()->id;
            $cancelledBooking->cancellation_reason = $request->reason;
            $cancelledBooking->cancellation_date = date('Y-m-d');
            $cancelledBooking->cancellation_time = date('H:i:s');
            $cancelledBooking->save();

            $slotTime = TimeSlot::where("id", $booking->slot_id)->first();
            $timeFrom = strtotime($slotTime->time_from);
            $timeTo = strtotime($slotTime->time_to);
            $newFormatFrom = date('H:i', $timeFrom);
            $newFormatTo = date('H:i', $timeTo);
            $doctor = Doctor::where("object_id", $booking->object_id)->first();
            $userId = UserBookings::where("reservation_record_id", $booking->reservation_record_id)->first()->user_id;
            $user = User::where("id", $userId)->first();
            if (Auth::user()->id == $doctor->user_id)
                $cancelledBy = $doctor->full_name;
            else
                $cancelledBy = User::where("id", Auth::user()->id)->first()->full_name;


            $userNotificationMsg = $this->getNotificationTextDetails("cancel_booking", ["name" => $cancelledBy, "type" => $booking->is_online, "date" => $booking->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
            $userNotificationTitle = $userNotificationMsg['title'][$user->notification_lang];
            $userNotificationBody = $userNotificationMsg['body'][$user->notification_lang];
            $userNotification = new Notifications();
            $userNotification->title = $userNotificationTitle;
            $userNotification->body = $userNotificationBody;
            $userNotification->user_id = $userId;
            $userNotification->type = 1;
            $userNotification->status = 1;
            $userNotification->save();
            $userDeviceToken = $user->device_token;
            $userDeviceTokens = $user->user_device_token->pluck("device_token")->toArray();
            array_push($userDeviceTokens, $userDeviceToken);
            $this->sendFirebaseNotificationCustom(["title" => $userNotificationTitle, "body" => $userNotificationBody], $userDeviceTokens);
            $userNot = Notifications::find($userNotification->id);
            $userNot->is_sent = 1;
            $userNot->save();

            $checkDoctor = User::where("id", $doctor->user_id);
            if($checkDoctor->exists()){
            $doctorNotificationLang = User::where("id", $doctor->user_id)->first()->notification_lang;
            $doctorNotificationMsg = $this->getNotificationTextDetails("cancel_booking", ["name" => $cancelledBy, "type" => $booking->is_online, "date" => $booking->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
            $doctorNotificationTitle = $doctorNotificationMsg['title'][$doctorNotificationLang];
            $doctorNotificationBody = $doctorNotificationMsg['body'][$doctorNotificationLang];

            $doctorNotification = new Notifications();
            $doctorNotification->title = $doctorNotificationTitle;
            $doctorNotification->body = $doctorNotificationBody;
            $doctorNotification->user_id = $doctor->user_id;
            $doctorNotification->type = 1;
            $doctorNotification->status = 1;
            $doctorNotification->save();
            //  $userDevicetoken = $this->getTokens(User::findMany($userId));
            //  $doctorDevicetoken = $this->getTokens(User::findMany($doctor->user_id));
            /*  $otherUserDevicesToken = UserDeviceToken::where("user_id", $userId)->get("device_token");
              if(count($otherUserDevicesToken) > 0){
                  for($i=0; $i<count($otherUserDevicesToken); $i++){
                      $userDevicetoken[] = $otherUserDevicesToken[$i]->device_token;
                  }
              }*/
            /* $otherDoctorDevicesToken = UserDeviceToken::where("user_id", $doctor->user_id)->get("device_token");
             if(count($otherDoctorDevicesToken) > 0){
                 for($i=0; $i<count($otherDoctorDevicesToken); $i++){
                     $doctorDevicetoken[] = $otherDoctorDevicesToken[$i]->device_token;
                 }
             }*/
                $checkDoctor = $checkDoctor->first();
                $doctorDeviceToken = $checkDoctor->device_token;
                $doctorDeviceTokens = $checkDoctor->user_device_token->pluck("device_token")->toArray();
                array_push($doctorDeviceTokens,$doctorDeviceToken);
            $this->sendFirebaseNotificationCustom(["title" => $doctorNotificationTitle, "body" => $doctorNotificationBody], $doctorDeviceTokens);
            $doctorNot = Notifications::find($doctorNotification->id);
            $doctorNot->is_sent = 1;
            $doctorNot->save();
        }
            DB::commit();
            return response()->json([
                "status" => true,
                "message" => "Booking cancelled successfully"
            ]);
        }else
            return response()->json([
                "status" => false,
                "message" => $checkCancellation["message"]
            ]);
        } catch (\Exception $e) {
            DB::rollback();
            throw $e;
        }
    }

    public function addRating(Request $request)
    {
//        return Auth::user();
        if (is_null($request->value)){
            if($request->lang == "ar")
                $msg = "قيمة التقييم مطلوبة.";
            else
                $msg = "The rating value field is required.";
            return response()->json([
                "status" => false,
                "message" => $msg
            ]);
        }
        $checkRating = DoctorUserRate::where([["rateable_type", "App\Models\User"], ["rateable_id", Auth::user()->id], ["rated_id", $request->doctor_id]])->exists();
        if($checkRating){
            if($request->lang == "ar")
                $msg = "لقد قمت بتقييم هذا الطبيب مسبقاً";
            else
                $msg = "You have already rated this doctor";
            return response()->json([
                "status" => false,
                "message" => $msg
            ]);
        }
        $user =Auth::user();
        $user->rateable()->create([
            "rated_id"=>$request->doctor_id,
            "rate"=>$request->value,
            "note"=>$request->notes
        ]);
        return response()->json([
            "status" => true,
            "message" => "Rating added successfully"
        ]);
    }

    public function getRating()
    {
        $avg = DoctorUserRate::where([["rateable_type", "App\Models\Doctor"], ["rated_id", Auth::user()->id]])->pluck("rate")->avg();
        if(is_null($avg))
            return response()->json([
                "status" => false,
                "message" => "There is no ratings"
            ]);
        $avg = round($avg * 2) / 2;
        return response()->json([
            "status" => true,
            "data" => $avg
        ]);
    }

    public function addDoctorRating(Request $request)
    {
        if (is_null($request->value)){
            if($request->lang == "ar")
                $msg = "قيمة التقييم مطلوبة.";
            else
                $msg = "The rating value field is required.";
            return response()->json([
                "status" => false,
                "message" => $msg
            ]);
        }
        $checkRating = DoctorUserRate::where([["rateable_type", "App\Models\User"], ["rateable_id", Auth::user()->id], ["rated_id", $request->doctor_id]])->exists();
        if($checkRating){
            if($request->lang == "ar")
                $msg = "لقد قمت بتقييم هذا الطبيب مسبقاً";
            else
                $msg = "You have already rated this doctor";
            return response()->json([
                "status" => false,
                "message" => $msg
            ]);
        }
        $user = User::find(Auth::user()->id);
        $user->rateable()->create([
            "rated_id"=>$request->doctor_id,
            "rate"=>$request->value,
            "note"=>$request->notes
        ]);
        return response()->json([
            "status" => true,
            "message" => "Rating added successfully"
        ]);
    }

    public function geDoctorRating()
    {
        dd("////");
        $user = User::find(1);

        return $user->rateable;
    }
}
