<?php

namespace App\Http\Controllers\Web\Admin;

use App\Helpers\ApiResponse\Json\JsonResponse;
use App\Http\Controllers\Controller;
use App\Models\CancelledBooking;
use App\Models\Date;
use App\Models\Doctor;
use App\Models\ObjectBooking;
use App\Models\ObjectWeekDays;
use App\Models\OnlineBooking;
use App\Models\TimeSlot;
use App\Models\User;
use App\Models\UserBookings;
use App\Repositories\AppointmentRepository;
use App\Traits\Firebase;
use App\Traits\IntegrationTrait;
use App\Traits\Notifications as NotificationsTrait;
use App\Traits\ZoomMeetingTrait;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Js;

class DateController extends Controller
{
    use IntegrationTrait;
    use Firebase;
    use ZoomMeetingTrait;
    use NotificationsTrait;
    const MEETING_TYPE_INSTANT = 1;
    const MEETING_TYPE_SCHEDULE = 2;
    const MEETING_TYPE_RECURRING = 3;
    const MEETING_TYPE_FIXED_RECURRING_FIXED = 8;

    private $repository;
    public function __construct(AppointmentRepository $repository){
        $this->repository = $repository;
    }
    public function index(Request $request)
    {

        $this->repository->index($request);
        $data['users_booking'] = UserBookings::join('object_bookings','user_bookings.reservation_record_id','=','object_bookings.reservation_record_id')
            ->join('doctors','object_bookings.object_id','=','doctors.object_id')
            ->get(['user_bookings.*','object_bookings.*','doctors.full_name','doctors.phone_number','doctors.email','doctors.specialization_id','doctors.has_zoom','doctors.object_id']);
        return view('admin.date.index',$data);
    }

    public function create(){
        $data['doctors']= Doctor::all();
        $data['users'] = User::all();
        return view('admin.date.create',$data);
    }
    public function store(Request $request){
        if($request->full_name && $request->email) {
            $user_booking = new UserBookings();
            $user_booking->user_id = null;
            $user_booking->reservation_record_id = null;
            $user_booking->full_name = $request->full_name;
            $user_booking->email = $request->email;
            $user_booking->from_dash = 1;
            if($user_booking->save()) {
                $user_booking = UserBookings::find($user_booking->id);
                $user_booking->reservation_record_id = $user_booking->id;
                $user_booking->save();
                $object_booking = new ObjectBooking();
                $object_booking->reservation_record_id = $user_booking->reservation_record_id;
                $object_booking->date = $request->date;
                $object_booking->object_id = $request->object_id;
                $object_booking->slot_id = $request->slotTime;
                $object_booking->is_online = $request->is_online;
                $object_booking->is_cancelled = 0;
                $object_booking->save();
                $date = Carbon::parse($request->date)->format('Y-m-d');
                $slotTime = TimeSlot::where("id", $request->slotTime)->first();
                if ($request->is_online == "1") {
                    $startTime = $date . "T" . $slotTime->time_from;
                    $path = 'users/me/meetings';
                    $response = $this->zoomPost($path, [
                        'topic' => "New Meeting",
                        'type' => self::MEETING_TYPE_SCHEDULE,
                        'start_time' => $this->toZoomTimeFormat($startTime),
                        'duration' => 30,
                        'agenda' => "agenda",
                        'timezone' => 'Asia/Amman',
                        'settings' => [
                            //   'alternative_hosts' => 'rawan_almasri@digisolfze.com',
                            'host_video' => false,
                            'participant_video' => false,
                            'waiting_room' => false,
                        ]
                    ]);
                    $meetingData = json_decode($response->body(), true);
                    $meeting = new OnlineBooking();
                    $meeting->booking_id = $object_booking->id;
                    $meeting->zoom_url = $meetingData["join_url"];
                    $meeting->meeting_id = $meetingData["id"];
                    $meeting->save();
                }
            }
        }else if ($request->user){
            $user_booking = new UserBookings();
            $user_booking->user_id = $request->user;
            $user_booking->reservation_record_id = null;
            $user_booking->full_name = null;
            $user_booking->email = null;
            $user_booking->from_dash = 1;
            if($user_booking->save()) {
                $user_booking = UserBookings::find($user_booking->id);
                $user_booking->reservation_record_id = $user_booking->id;
                $user_booking->save();
                $object_booking = new ObjectBooking();
                $object_booking->reservation_record_id = $user_booking->reservation_record_id;
                $object_booking->date = $request->date;
                $object_booking->object_id = $request->object_id;
                $object_booking->slot_id = $request->slotTime;
                $object_booking->is_online = $request->is_online;
                $object_booking->is_cancelled = 0;
                $object_booking->save();
                $date = Carbon::parse($request->date)->format('Y-m-d');
                $slotTime = TimeSlot::where("id", $request->slotTime)->first();
                if ($request->is_online == "1") {
                    $startTime = $date . "T" . $slotTime->time_from;
                    $path = 'users/me/meetings';
                    $response = $this->zoomPost($path, [
                        'topic' => "New Meeting",
                        'type' => self::MEETING_TYPE_SCHEDULE,
                        'start_time' => $this->toZoomTimeFormat($startTime),
                        'duration' => 30,
                        'agenda' => "agenda",
                        'timezone' => 'Asia/Amman',
                        'settings' => [
                            //   'alternative_hosts' => 'rawan_almasri@digisolfze.com',
                            'host_video' => false,
                            'participant_video' => false,
                            'waiting_room' => false,
                        ]
                    ]);
                    $meetingData = json_decode($response->body(), true);
                    $meeting = new OnlineBooking();
                    $meeting->booking_id = $object_booking->id;
                    $meeting->zoom_url = $meetingData["join_url"];
                    $meeting->meeting_id = $meetingData["id"];
                    $meeting->save();
                }
            }

        }
        return redirect()->route('admin.date.index');

    }

    public function details(Request $request){
        if ($request->id  && $request->reservation_record_id && $request->object_id) {
            $data['doctor'] = Doctor::where('object_id',$request->object_id)->first();
            $data['users_booking'] = UserBookings::where('user_bookings.reservation_record_id', $request->reservation_record_id)
                ->join('object_bookings', 'user_bookings.reservation_record_id', '=', 'object_bookings.reservation_record_id')
                ->join('doctors', 'object_bookings.object_id', '=', 'doctors.object_id')
                ->first(['user_bookings.*','object_bookings.*','doctors.full_name','doctors.has_zoom','doctors.email','doctors.phone_number']);
            $data['user_id'] =  UserBookings::where('user_bookings.reservation_record_id', $request->reservation_record_id)->first();
            if ($data['user_id'])$data['user'] = User::where('id',$data['user_id']->user_id)->first();
        }
        return view('admin.date.details',$data);
    }

    public function edit(Request $request)
    {
        $data['reservation_record_id'] = $request->reservation_record_id;
        $data['user_object_booking'] = ObjectBooking::find($request->reservation_record_id);
        $data['user_slot_time'] = TimeSlot::where('id', $data['user_object_booking']->slot_id)->first();
        $data['user_doctor'] = Doctor::where('id', $data['user_object_booking']->object_id)->first();
        $data['object_week_days'] = ObjectWeekDays::where('id', $data['user_slot_time']->object_week_days_id)->first();
        $data['doctors'] = Doctor::all();
        $data['slot_time'] = TimeSlot::all();
        return view('admin.date.edit',$data);
    }


    public function update(Request $request)
    {
        $this->repository->update($request);
        return redirect()->route('admin.date.index');

    }
}
