<?php

namespace App\Http\Controllers\Web\Admin;

use App\Helpers\Dialog\Web\Dialog;
use App\Helpers\Dialog\Web\Types\DangerMessage;
use App\Helpers\Dialog\Web\Types\SuccessMessage;
use App\Http\Controllers\Controller;
use App\Models\Doctor;
use App\Models\Specialization;
use App\Models\TimeSlotType;
use App\Models\User;
use App\Rules\AlphaSpace;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Traits\IntegrationTrait;

class DoctorController extends Controller
{
    use IntegrationTrait;
    public  function rules(){
        return [
            "full_name" => ["required",new AlphaSpace(), "max:255"],
            "phone_number" => ["required", "max:255"],
            "email" => ["required","email"],
            "specialization" => ["required"],
            "time_slot_type" => ["required"]
        ];
    }
    public  function updateRules(){
        return [
            "full_name" => ["required",new AlphaSpace(), "max:255"],
            "phone_number" => ["required", "max:255"],
            "email" => ["required","email"]
        ];
    }

    public function index()
    {
        $data['doctors'] = Doctor::all();
        for($i=0; $i<count($data['doctors']); $i++){
            $data['doctors'][$i]["specialization_en"] = Specialization::where("id", $data['doctors'][$i]["specialization_id"])->first()->name_en;
            $data['doctors'][$i]["specialization_ar"] = Specialization::where("id", $data['doctors'][$i]["specialization_id"])->first()->name_ar;
            $imageData = $data["doctors"][$i]->getFirstMediaFile("Doctors");
            $data["doctors"][$i]["image"] = $imageData != null  ? $imageData["path"]."/".$imageData["filename"] : null;
        }
        return view("admin.doctor.index", $data);
    }

    public function create()
    {
        $data["specializations"] = Specialization::all();
        $data["timeSlotTypes"] = TimeSlotType::all();
        $data["users"] = User::all();
        return view("admin.doctor.create", $data);
    }

    public function store(Request $request)
    {
        $valid = Validator::make($request->all(), $this->rules());
        if($valid->fails()){
            return redirect()->route("admin.doctor.create")->withInput($request->all())->withErrors($valid->errors()->messages());
        }
        $doctor = new Doctor();
        $doctor->full_name = $request->full_name;
        $doctor->phone_number = $request->phone_number;
        $doctor->email = $request->email;
        $doctor->specialization_id = $request->specialization;
        $doctor->user_id = $request->user_id ?? null;
        $doctor->has_zoom = isset($request->has_zoom) ? 1 : 0;
        $doctor->zoom_email = $request->zoom_email;
        $doctor->zoom_password = $request->zoom_password;
        $doctor->description = $request->doctor_description ?? null;
       if($doctor->save()){
           if($request->hasFile("doctor_image")){
               $doctor->saveMedia($request->file("doctor_image"),'Doctors');
           }
           $response = $this->addNewObject($request);
           if($response->getData()->status){
                $doctor = Doctor::findOrFail($doctor->id);
                $doctor->object_id = $response->getData()->objectId;
                $doctor->save();
            }else{
                $message = (new DangerMessage())->title("Create Failed")
                    ->body("Failed In Create Object");
                Dialog::flashing($message);
                return redirect()->route("admin.doctor.index");
            }
        }
        $message = (new SuccessMessage())->title("Create Successfully")
            ->body("The Doctor Has Been Create Successfully");
        Dialog::flashing($message);
        return redirect()->route("admin.doctor.index");
    }

    public function edit(Request $request)
    {
        $objectDetails = $this->getObjectInfo($request);
        $timeSlotTypes = $this->getAllTimeSlotTypes();
        $data['doctor'] = Doctor::find($request->id);
        $data['objectDetails'] = $objectDetails->getData()->data;
        $data["specializations"] = Specialization::all();
        $data["timeSlotTypes"] = $timeSlotTypes->getData()->data;
        $data["users"] = User::all();
        return view("admin.doctor.edit", $data);
    }

    public function update(Request $request)
    {

        $valid = Validator::make($request->all(), $this->updateRules());
        if($valid->fails()){
            return redirect()->route("admin.doctor.edit", $request->id)->withInput($request->all())->withErrors($valid->errors()->messages());
        }
        $doctor = Doctor::findOrFail($request->id);
        $doctor->full_name = $request->full_name;
        $doctor->phone_number = $request->phone_number;
        $doctor->email = $request->email;
        $doctor->specialization_id = $request->specialization;
        $doctor->user_id = $request->user_id ?? null;
        $doctor->has_zoom = isset($request->has_zoom) ? 1 : 0;
        $doctor->zoom_email = $request->zoom_email;
        $doctor->zoom_password = $request->zoom_password;
        $doctor->description = $request->doctor_description;
        $doctor->save();
        if($request->hasFile("doctor_image")){
            $doctor->removeAllFiles();
            $doctor->saveMedia($request->file("doctor_image"),'Doctors');
        }
        $this->updateObject($request);
        $message = (new SuccessMessage())->title("Updated Successfully")
            ->body("The Doctor Has Been Updated Successfully");
        Dialog::flashing($message);
        return redirect()->route("admin.doctor.index");
    }

    public function cancel(){
        return redirect()->route("admin.doctor.index");
    }
    public function destroy(Request $request)
    {
        $doctor = Doctor::find($request->id);
        if($doctor->delete()){
            $doctor->removeAllFiles();
        }
        $message = (new DangerMessage())->title("Deleted Successfully")
            ->body("The Doctor Has Been Deleted Successfully");
        Dialog::flashing($message);
        return redirect()->route("admin.doctor.index");
    }
}
