<?php

namespace App\Http\Livewire;
use App\Models\Doctor;
use App\Models\ObjectBooking;
use App\Models\OnlineBooking;
use App\Models\TimeSlot;
use App\Models\User;
use App\Models\UserBookings;
use App\Traits\Firebase;
use App\Traits\IntegrationTrait;
use App\Traits\Notifications as NotificationsTrait;
use App\Traits\ZoomMeetingTrait;
use Livewire\Component;
use Carbon\Carbon;
use Illuminate\Http\Request;

class CreateExistUserAppointment extends Component
{
    use IntegrationTrait;
    use Firebase;
    use ZoomMeetingTrait;
    use NotificationsTrait;
    const MEETING_TYPE_INSTANT = 1;
    const MEETING_TYPE_SCHEDULE = 2;
    const MEETING_TYPE_RECURRING = 3;
    const MEETING_TYPE_FIXED_RECURRING_FIXED = 8;
    public $full_name,
           $email,
           $users,
           $user,
           $object_id,
           $is_online,
           $user_object_booking,
           $slotTime,
           $doctors,
           $slot_time,
           $days;
    protected function rules()
    {
        return [
            'user' =>'required',
            'object_id' => 'required',
            'is_online' => 'required',
            'user_object_booking' => 'required',
            'slotTime' => 'required',
        ];
    }
    public function updated($fields)
    {
        $this->validateOnly($fields);
    }
    public function handleTime(Request $request){
        $dt = strtotime($this->user_object_booking);
        $this->days = date("l", $dt);
        $this->slot_time = TimeSlot::all();

    }
    public function doctorData(){
        $this->doctors = Doctor::all();
    }
    public function userData(){
        $this->users = User::all();
    }
    public function store(Request $request){
        $validated = $this->validate();
        if($validated) {
                $user_booking = new UserBookings();
                $user_booking->user_id = $this->user;
                $user_booking->reservation_record_id = null;
                $user_booking->full_name = null;
                $user_booking->email = null;
                $user_booking->from_dash = 1;
                if($user_booking->save()) {
                    $user_booking = UserBookings::find($user_booking->id);
                    $user_booking->reservation_record_id = $user_booking->id;
                    $user_booking->save();
                    $object_booking = new ObjectBooking();
                    $object_booking->reservation_record_id = $user_booking->reservation_record_id;
                    $object_booking->date = $this->user_object_booking;
                    $object_booking->object_id = $this->object_id;
                    $object_booking->slot_id = $this->slotTime;
                    $object_booking->is_online = $this->is_online;
                    $object_booking->is_cancelled = 0;
                    $object_booking->save();
                    $date = Carbon::parse($this->user_object_booking)->format('Y-m-d');
                    $slotTime = TimeSlot::where("id", $this->slotTime)->first();
                    if ($this->is_online == "1") {
                        $startTime = $date . "T" . $slotTime->time_from;
                        $path = 'users/me/meetings';
                        $response = $this->zoomPost($path, [
                            'topic' => "New Meeting",
                            'type' => self::MEETING_TYPE_SCHEDULE,
                            'start_time' => $this->toZoomTimeFormat($startTime),
                            'duration' => 30,
                            'agenda' => "agenda",
                            'timezone' => 'Asia/Amman',
                            'settings' => [
                                'host_video' => false,
                                'participant_video' => false,
                                'waiting_room' => false,
                            ]
                        ]);
                        $meetingData = json_decode($response->body(), true);
                        $meeting = new OnlineBooking();
                        $meeting->booking_id = $object_booking->id;
                        $meeting->zoom_url = $meetingData["join_url"];
                        $meeting->meeting_id = $meetingData["id"];
                        $meeting->save();
                    }
            }
            $this->dispatchBrowserEvent('close-exist-user-modal');
            $this->reset(['user','object_id','is_online','user_object_booking','slotTime']);
        }
    }
    public function render()
    {
        return view('livewire.create-exist-user-appointment');
    }
}
