<?php

namespace App\Repositories;

use App\Models\CancelledBooking;
use App\Models\Doctor;
use App\Models\Notifications;
use App\Models\ObjectBooking;
use App\Models\TimeSlot;
use App\Models\User;
use App\Models\UserBookings;
use App\Models\UserDeviceToken;
use App\Traits\Firebase;
use App\Traits\IntegrationTrait;
use App\Traits\Notifications as NotificationsTrait;
use Illuminate\Support\Facades\Auth;

class AppointmentRepository
{
    use IntegrationTrait;
    use NotificationsTrait;
    use Firebase;
    public function rules(){
        return [
            'canselReason' => ['required']
        ];
    }
    public function index($request){
        if($request->statusId) {

            if ($request->statusId == 3) {
                $cancelledBooking = CancelledBooking::where('booking_id',$request->bookingId)->first();
                if ($cancelledBooking) {
                    $cancelledBooking->delete();
                }
                $booking = ObjectBooking::where('reservation_record_id',$request->bookingId)->first();
                $booking->is_cancelled = 0;
                $booking->save();
                $slotTime = TimeSlot::where("id", $booking->slot_id)->first();
                $userId = UserBookings::where("reservation_ record_id", $booking->reservation_record_id)->first()->user_id;
                $timeFrom = strtotime($slotTime->time_from);
                $timeTo = strtotime($slotTime->time_to);
                $newFormatFrom = date('H:i', $timeFrom);
                $newFormatTo = date('H:i', $timeTo);
                $doctor = Doctor::where("object_id", $booking->object_id)->first();
                $userId = UserBookings::where("reservation_record_id", $booking->reservation_record_id)->first()->user_id;
                $user = User::where("id", $userId)->first();
                if (Auth::user()->id == $doctor->user_id)
                    $activeBy = $doctor->full_name;
                else
                    $activeBy = Auth::user()->full_name;

                $doctorNotificationLang = User::where("id", $doctor->user_id)->first()->notification_lang;
                $userNotificationMsg = $this->getNotificationTextDetails("UnCancelled_booking", ["name" => $activeBy, "type" => $booking->is_online, "date" => $booking->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
                $doctorNotificationMsg = $this->getNotificationTextDetails("UnCancelled_booking", ["name" => $activeBy, "type" => $booking->is_online, "date" => $booking->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
                $userNotificationTitle = $userNotificationMsg['title'][$user->notification_lang];
                $userNotificationBody = $userNotificationMsg['body'][$user->notification_lang];
                $doctorNotificationTitle = $doctorNotificationMsg['title'][$doctorNotificationLang];
                $doctorNotificationBody = $doctorNotificationMsg['body'][$doctorNotificationLang];
                $userNotification = new Notifications();
                $userNotification->title = $userNotificationTitle;
                $userNotification->body = $userNotificationBody;
                $userNotification->user_id = $userId;
                $userNotification->type = 1;
                $userNotification->status = 1;
                $userNotification->save();
                $doctorNotification = new Notifications();
                $doctorNotification->title = $doctorNotificationTitle;
                $doctorNotification->body = $doctorNotificationBody;
                $doctorNotification->user_id = $doctor->user_id;
                $doctorNotification->type = 1;
                $doctorNotification->status = 1;
                $doctorNotification->save();
                $userDevicetoken = $this->getTokens(User::findMany($userId));
                $doctorDevicetoken = $this->getTokens(User::findMany($doctor->user_id));
                $otherUserDevicesToken = UserDeviceToken::where("user_id", $userId)->get("device_token");
                if (count($otherUserDevicesToken) > 0) {
                    for ($i = 0; $i < count($otherUserDevicesToken); $i++) {
                        $userDevicetoken[] = $otherUserDevicesToken[$i]->device_token;
                    }
                }
                $otherDoctorDevicesToken = UserDeviceToken::where("user_id", $doctor->user_id)->get("device_token");
                if (count($otherDoctorDevicesToken) > 0) {
                    for ($i = 0; $i < count($otherDoctorDevicesToken); $i++) {
                        $doctorDevicetoken[] = $otherDoctorDevicesToken[$i]->device_token;
                    }
                }
                $this->sendFirebaseNotificationCustom(["title" => $userNotificationTitle, "body" => $userNotificationBody], $userDevicetoken);
                $userNot = Notifications::find($userNotification->id);
                $userNot->is_sent = 1;
                $userNot->save();
                $this->sendFirebaseNotificationCustom(["title" => $doctorNotificationTitle, "body" => $doctorNotificationBody], $doctorDevicetoken);
                $doctorNot = Notifications::find($doctorNotification->id);
                $doctorNot->is_sent = 1;
                $doctorNot->save();
            }
            if ($request->statusId == 1) {

                $booking = ObjectBooking::where('reservation_record_id',$request->bookingId)->first();
                $booking->is_cancelled = 1;
                $booking->save();
                $cancelledBooking = CancelledBooking::find($request->bookingId);
                if ($cancelledBooking) {
                    $cancelledBooking->delete();
                }
                $cancelledBooking = new CancelledBooking();
                $cancelledBooking->booking_id = (int)$booking->reservation_record_id;
                $cancelledBooking->cancelled_by = Auth::user()->id;
                $cancelledBooking->cancellation_reason = $request->canselReason;
                $cancelledBooking->cancellation_date = date('Y-m-d');
                $cancelledBooking->cancellation_time = date('H:i:s');
                $cancelledBooking->save();
                $slotTime = TimeSlot::where("id", $booking->slot_id)->first();
                $userId = UserBookings::where("reservation_record_id", $booking->reservation_record_id)->first()->user_id;
                $timeFrom = strtotime($slotTime->time_from);
                $timeTo = strtotime($slotTime->time_to);
                $newFormatFrom = date('H:i', $timeFrom);
                $newFormatTo = date('H:i', $timeTo);
                $doctor = Doctor::where("object_id", $booking->object_id)->first();
                $userId = UserBookings::where("reservation_record_id", $booking->reservation_record_id)->first()->user_id;
                $user = User::where("id", $userId)->first();
                if (Auth::user()->id == $doctor->user_id)
                    $cancelledBy = $doctor->full_name;
                else
                    $cancelledBy = Auth::user()->full_name;
                if ($doctor->user_id) {
                    $doctorNotificationLang = User::where("id", $doctor->user_id)->first()->notification_lang;
                    $userNotificationMsg = $this->getNotificationTextDetails("cancel_booking", ["name" => $cancelledBy, "type" => $booking->is_online, "date" => $booking->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
                    $doctorNotificationMsg = $this->getNotificationTextDetails("cancel_booking", ["name" => $cancelledBy, "type" => $booking->is_online, "date" => $booking->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
                    $userNotificationTitle = $userNotificationMsg['title'][$user->notification_lang];
                    $userNotificationBody = $userNotificationMsg['body'][$user->notification_lang];
                    $doctorNotificationTitle = $doctorNotificationMsg['title'][$doctorNotificationLang];
                    $doctorNotificationBody = $doctorNotificationMsg['body'][$doctorNotificationLang];
                    $userNotification = new Notifications();
                    $userNotification->title = $userNotificationTitle;
                    $userNotification->body = $userNotificationBody;
                    $userNotification->user_id = $userId;
                    $userNotification->type = 1;
                    $userNotification->status = 1;
                    $userNotification->save();
                    $doctorNotification = new Notifications();
                    $doctorNotification->title = $doctorNotificationTitle;
                    $doctorNotification->body = $doctorNotificationBody;
                    $doctorNotification->user_id = $doctor->user_id;
                    $doctorNotification->type = 1;
                    $doctorNotification->status = 1;
                    $doctorNotification->save();
                    $userDevicetoken = $this->getTokens(User::findMany($userId));
                    $doctorDevicetoken = $this->getTokens(User::findMany($doctor->user_id));
                    $otherUserDevicesToken = UserDeviceToken::where("user_id", $userId)->get("device_token");
                    if (count($otherUserDevicesToken) > 0) {
                        for ($i = 0; $i < count($otherUserDevicesToken); $i++) {
                            $userDevicetoken[] = $otherUserDevicesToken[$i]->device_token;
                        }
                    }
                    $otherDoctorDevicesToken = UserDeviceToken::where("user_id", $doctor->user_id)->get("device_token");
                    if (count($otherDoctorDevicesToken) > 0) {
                        for ($i = 0; $i < count($otherDoctorDevicesToken); $i++) {
                            $doctorDevicetoken[] = $otherDoctorDevicesToken[$i]->device_token;
                        }
                    }
                    $this->sendFirebaseNotificationCustom(["title" => $userNotificationTitle, "body" => $userNotificationBody], $userDevicetoken);
                    $userNot = Notifications::find($userNotification->id);
                    $userNot->is_sent = 1;
                    $userNot->save();
                    $this->sendFirebaseNotificationCustom(["title" => $doctorNotificationTitle, "body" => $doctorNotificationBody], $doctorDevicetoken);
                    $doctorNot = Notifications::find($doctorNotification->id);
                    $doctorNot->is_sent = 1;
                    $doctorNot->save();
                }

            }
        }
    }

    public function update($request){
        $data['user_object_booking'] = ObjectBooking::find($request->reservation_record_id);
        if ($request->date){
            $data['object_booking'] = ObjectBooking::where('slot_id',$request->slotTime)->count();
            $this->updateCondition($request,$data['user_object_booking'],$data['object_booking']);
        }else{
            $this->updateCondition($request,$data['user_object_booking'],0);
        }
        $slotTime = TimeSlot::where("id", $data['user_object_booking']->slot_id)->first();
        $userId = UserBookings::where("reservation_record_id", $data['user_object_booking']->reservation_record_id)->first()->user_id;
        $timeFrom = strtotime($slotTime->time_from);
        $timeTo = strtotime($slotTime->time_to);
        $newFormatFrom = date('H:i', $timeFrom);
        $newFormatTo = date('H:i', $timeTo);
        $doctor = Doctor::where("object_id", $data['user_object_booking']->object_id)->first();
        $userId = UserBookings::where("reservation_record_id", $data['user_object_booking']->reservation_record_id)->first()->user_id;
        $user = User::where("id", $userId)->first();
        if (Auth::user()->id == $doctor->user_id)
            $activeBy = $doctor->full_name;
        else
            $activeBy = Auth::user()->full_name;

        $doctorNotificationLang = User::where("id", $doctor->user_id)->first()->notification_lang;
        $userNotificationMsg = $this->getNotificationTextDetails("Edit_booking", ["name" => $activeBy, "type" => $data['user_object_booking']->is_online, "date" => $data['user_object_booking']->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
        $doctorNotificationMsg = $this->getNotificationTextDetails("Edit_booking", ["name" => $activeBy, "type" => $data['user_object_booking']->is_online, "date" => $data['user_object_booking']->date, "from" => $newFormatFrom, "to" => $newFormatTo]);
        $doctorNotificationTitle = $doctorNotificationMsg['title'][$doctorNotificationLang];
        $doctorNotificationBody = $doctorNotificationMsg['body'][$doctorNotificationLang];
        $userNotificationTitle = $userNotificationMsg['title'][$user->notification_lang];
        $userNotificationBody = $userNotificationMsg['body'][$user->notification_lang];
        $userNotification = new Notifications();
        $userNotification->title = $userNotificationTitle;
        $userNotification->body = $userNotificationBody;
        $userNotification->user_id = $userId;
        $userNotification->type = 1;
        $userNotification->status = 1;
        $userNotification->save();
        $doctorNotification = new Notifications();
        $doctorNotification->title = $doctorNotificationTitle;
        $doctorNotification->body = $doctorNotificationBody;
        $doctorNotification->user_id = $doctor->user_id;
        $doctorNotification->type = 1;
        $doctorNotification->status = 1;
        $doctorNotification->save();
        $userDevicetoken = $this->getTokens(User::findMany($userId));
        $doctorDevicetoken = $this->getTokens(User::findMany($doctor->user_id));
        $otherUserDevicesToken = UserDeviceToken::where("user_id", $userId)->get("device_token");
        if (count($otherUserDevicesToken) > 0) {
            for ($i = 0; $i < count($otherUserDevicesToken); $i++) {
                $userDevicetoken[] = $otherUserDevicesToken[$i]->device_token;
            }
        }
        $otherDoctorDevicesToken = UserDeviceToken::where("user_id", $doctor->user_id)->get("device_token");
        if (count($otherDoctorDevicesToken) > 0) {
            for ($i = 0; $i < count($otherDoctorDevicesToken); $i++) {
                $doctorDevicetoken[] = $otherDoctorDevicesToken[$i]->device_token;
            }
        }
        $this->sendFirebaseNotificationCustom(["title" => $userNotificationTitle, "body" => $userNotificationBody], $userDevicetoken);
        $userNot = Notifications::find($userNotification->id);
        $userNot->is_sent = 1;
        $userNot->save();
        $this->sendFirebaseNotificationCustom(["title" => $doctorNotificationTitle, "body" => $doctorNotificationBody], $doctorDevicetoken);
        $doctorNot = Notifications::find($doctorNotification->id);
        $doctorNot->is_sent = 1;
        $doctorNot->save();
    }
    public function updateCondition($request,$user_object_booking,$object_booking){
        if ($user_object_booking || $object_booking == 0){
            $user_object_booking->slot_id = $request->slotTime ?? $user_object_booking->slot_id;
            if ($request->object_id) {
                $doctor_object_booking = ObjectBooking::where('object_id',$request->object_id)->get();
                foreach ($doctor_object_booking as $doctor){
                    if($doctor->slot_id && $doctor->date){
                        return redirect()->back();
                    }
                }
            }else {
                $user_object_booking->object_id = $request->object_id;
                $user_object_booking->date = $request->date ?? $user_object_booking->date;
                $user_object_booking->save();
                return redirect()->route('admin.date.index');
            }
        }else{
            return redirect()->back();
        }
    }
}
